@@ -126,7 +126,6 @@ struct _HildonFileChooserDialogPrivate {
     GtkSizeGroup *caption_size_group;
     GtkWidget *caption_control_name;
     GtkWidget *caption_control_location;
-    GtkWidget *label_location;
     GtkWidget *entry_name;
     GtkWidget *hbox_location, *image_location, *title_location;
     GtkWidget *extensions_combo;
@@ -706,6 +705,9 @@ check_for_location_update (GtkTreeModel *model,
   if (current_path
       && gtk_tree_path_compare (path, current_path) == 0) 
     hildon_file_chooser_dialog_update_location_info(priv);
+
+  if (current_path)
+    gtk_tree_path_free (current_path);
 }
 
 static gboolean
@@ -1066,8 +1068,9 @@ static void build_ui(HildonFileChooserDialog * self)
                                    _("ckdg_fi_save_object_name"));
         }
 
-        gtk_label_set_text(GTK_LABEL(priv->label_location),
-                                     _("sfil_fi_save_objects_location"));
+
+		hildon_caption_set_label (HILDON_CAPTION (priv->caption_control_location),
+				_("sfil_fi_save_objects_location"));
         gtk_widget_hide(GTK_WIDGET(priv->filetree));
 
         /* Content pane of the filetree widget needs to be realized.
@@ -1108,8 +1111,8 @@ static void build_ui(HildonFileChooserDialog * self)
     case GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER:
         hildon_caption_set_label(HILDON_CAPTION(priv->caption_control_name),
                                  _("ckdg_fi_new_folder_name"));
-        gtk_label_set_text(GTK_LABEL(priv->label_location),
-                                     _("ckdg_fi_new_folder_location"));
+		hildon_caption_set_label (HILDON_CAPTION (priv->caption_control_location),
+						 _("ckdg_fi_new_folder_location"));
         gtk_widget_show_all(priv->caption_control_name);
         gtk_widget_hide(GTK_WIDGET(priv->filetree));
         _hildon_file_selection_realize_help(priv->filetree);
@@ -1338,6 +1341,22 @@ static void handle_folder_popup(HildonFileChooserDialog *self)
   g_free(uri);
 }
 
+static gboolean
+on_location_hw_enter_pressed (GtkWidget * widget, GdkEventKey * event,
+			      gpointer data)
+{
+  HildonFileChooserDialog *self = HILDON_FILE_CHOOSER_DIALOG(data);
+  switch (event->keyval) {
+  case GDK_Return:
+    if (self->priv->action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+	self->priv->action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER)
+      handle_folder_popup(self);
+    return TRUE;
+  default:
+    return FALSE;
+  }
+}
+
 static gboolean hildon_file_chooser_dialog_location_pressed(GtkWidget
   *widget, GdkEventButton *event, gpointer data)
 {
@@ -1973,17 +1992,19 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     gtk_event_box_set_visible_window(GTK_EVENT_BOX(eventbox), FALSE);
     gtk_container_add(GTK_CONTAINER(eventbox), priv->hbox_location);
     gtk_widget_add_events(eventbox, GDK_BUTTON_PRESS_MASK);
+    g_object_set (G_OBJECT (eventbox), "can-focus", TRUE, NULL);
+  
+    priv->caption_control_location =
+      hildon_caption_new(size_group, _("sfil_fi_save_objects_items"),
+			 eventbox, NULL,
+			 HILDON_CAPTION_OPTIONAL);
+    hildon_caption_set_separator(HILDON_CAPTION(priv->caption_control_location),
+                                 "");
 
-    priv->caption_control_location = gtk_hbox_new(FALSE, HILDON_MARGIN_DOUBLE);
-    priv->label_location = g_object_new(GTK_TYPE_LABEL, "label", _("sfil_fi_save_objects_location"), "xalign", 1.0f, NULL);
     label_items = g_object_new(GTK_TYPE_LABEL, "label",  _("sfil_fi_save_objects_items"), "xalign", 1.0f, NULL);
     priv->multiple_label = gtk_label_new(NULL);
     gtk_box_pack_start(GTK_BOX(priv->hbox_items), label_items, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(priv->hbox_items), priv->multiple_label, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(priv->caption_control_location),
-                       priv->label_location, FALSE, TRUE, 0);
-    gtk_box_pack_start(GTK_BOX(priv->caption_control_location), eventbox, TRUE, TRUE, 0);
-    gtk_size_group_add_widget(size_group, priv->label_location);
     gtk_size_group_add_widget(size_group, label_items);
 
     priv->popup = gtk_menu_new();
@@ -2056,6 +2077,8 @@ static void hildon_file_chooser_dialog_init(HildonFileChooserDialog * self)
     g_signal_connect(eventbox, "button-release-event",
                      G_CALLBACK(hildon_file_chooser_dialog_location_pressed),
                      self);
+    g_signal_connect (eventbox, "key-press-event", 
+		      G_CALLBACK (on_location_hw_enter_pressed), self);	
 
     gtk_dialog_set_default_response(GTK_DIALOG(self), GTK_RESPONSE_OK);
 }
